'use strict';

let checkPassword = require('bem-check-password');
let sha1 = require('bem-sha1');
let padnum = require('padnum');
let rid = require('bem-rid');

class Base {
  constructor(config) {
    this.db = config.db;
    this.className = config.className;
    this.mongo = config.mongo;
    this.secret = config.secret || 'bem';
  }

  error(code, message) {
    let err = new Error(message);
    err.code = code;
    throw err;
  }

  * isValid(userData) {
    return false;
  }

  newObjectId() {
    return rid({
      length: 12,
      radix: 62
    });
  }

  * addUser(userData) {
    userData.source = userData.source || '1';
    let isPasswordValid = this.checkPassword(userData.password);
    if (!isPasswordValid) {
      this.error(2, 'invalid password');
    }
    let isValid = yield this.isValid(userData);
    if (!isValid) {
      this.error(3, 'exists phone');
    }
    userData.password = sha1(userData.password);
    let objectId = this.newObjectId();
    Object.assign(userData, {
      objectId: objectId
    });
    let result = yield this.mongo.create({
      className: this.className,
      obj: userData
    });
    return result;
  }

  checkPassword(password) {
    return !checkPassword(password);
  }

}

module.exports = Base;
