'use strict';

let Captcha = require('bem-captcha');
let jwt = require('jsonwebtoken');

let Base = require('./base');

class Email extends Base {
  constructor(config) {
    super(config);
  }

  * isValid(userData) {
    let count = yield this.mongo.count({
      className: this.className,
      where: {
        email: userData.email
      }
    });
    return count === 0;
  }

  * getCaptcha(email) {
    let captcha = yield Captcha.get();
    return yield captcha.getCaptcha(email);
  }

  * verifyCaptcha(email, captchaNum) {
    let captcha = yield Captcha.get();
    let isCaptchaValid = yield captcha.verifyCaptcha(email, captchaNum);
    if (!isCaptchaValid) {
      this.error(1, 'invalid captcha');
    }
    let token = jwt.sign({
      email: email
    }, this.secret, {
      expiresIn: '10m'
    });
    return token;
  }

  * verifyCaptchaToken(email, token) {
    try {
      let data = jwt.verify(token, this.secret);
      return email && (email === data.email);
    } catch (e) {
      return false;
    }
  }

}

module.exports = Email;
