'use strict';

let Captcha = require('bem-captcha');
let jwt = require('jsonwebtoken');

let Base = require('./base');

class Phone extends Base {
  constructor(config) {
    super(config);
  }

  * isValid(userData) {
    let count = yield this.mongo.count({
      className: this.className,
      where: {
        phone: userData.phone
      }
    });
    return count === 0;
  }

  * getCaptcha(phone) {
    let captcha = yield Captcha.get();
    return yield captcha.getCaptcha(phone);
  }

  * verifyCaptcha(phone, captchaNum) {
    let captcha = yield Captcha.get();
    let isCaptchaValid = yield captcha.verifyCaptcha(phone, captchaNum);
    if (!isCaptchaValid) {
      this.error(1, 'invalid captcha');
    }
    let token = jwt.sign({
      phone: phone
    }, this.secret, {
      expiresIn: '10m'
    });
    return token;
  }

  * verifyCaptchaToken(phone, token) {
    try {
      let data = jwt.verify(token, this.secret);
      return phone && (phone === data.phone);
    } catch (e) {
      return false;
    }
  }

}

module.exports = Phone;
