'use strict'

var assert = require("assert");
var request = require('request');
var Register = require('../index');
var co = require('co');
var thunkify = require('thunkify');
var MongoClient = require('mongodb').MongoClient;
var util = require('util');
let jwt = require('jsonwebtoken');

var phone = '18392991829';

describe('Register', function() {
  let register, captcha, token, secret = 'haha';
  before(function(done) {
    co(function*() {
      let db;
      let constr = util.format('mongodb://%s/%s', 'localhost', 'test');
      db = yield thunkify(MongoClient.connect.bind(MongoClient))(constr, {
        db: {
          w: 1,
          readPreference: 'secondary'
        }
      });
      var collection = db.collection('_User');
      yield thunkify(collection.remove.bind(collection))({});

      register = yield Register.get({
        db: 'test',
        secret: secret
      });
    }).then(done, done);
  });

  describe('Register#isValid', function() {
    it('手机号可用', function(done) {
      co(function*() {
        let isValid = yield register.isValid({
          phone: phone
        });
        assert.equal(isValid, true);
      }).then(done, done);
    });
  });

  describe('Register#getCaptcha', function() {
    it('获取验证码', function(done) {
      co(function*() {
        let num = yield register.getCaptcha(phone);
        assert.equal(num.length, 6);
        captcha = num;
      }).then(done, done);
    });
  });

  describe('Register#verifyCaptcha', function() {

    it('错误验证码', function(done) {
      co(function*() {
        let err;
        try {
          yield register.verifyCaptcha(phone, '123');
        } catch (e) {
          err = e;
        }
        assert.equal(err.code, 1);
      }).then(done, done);
    });

    it('正确验证码', function(done) {
      co(function*() {
        token = yield register.verifyCaptcha(phone, captcha);
        let data = jwt.verify(token, secret);
        assert.equal(data.phone, phone);
      }).then(done, done);
    });

  });

  describe('Register#verifyCaptchaToken', function() {

    it('错误token', function(done) {
      co(function*() {
        let isValid = yield register.verifyCaptchaToken(phone, '123');
        assert.equal(isValid, false);
      }).then(done, done);
    });

    it('正确token', function(done) {
      co(function*() {
        let isValid = yield register.verifyCaptchaToken(phone, token);
        assert.equal(isValid, true);
      }).then(done, done);
    });

  });

  describe('Register#addUser', function() {

    it('密码不符合要求', function(done) {
      co(function*() {
        let err;
        try {
          yield register.addUser({
            phone: phone,
            nickname: 'seon',
            password: '123'
          });
        } catch (e) {
          err = e;
        }
        assert.equal(err.code, 2);
      }).then(done, done);
    });

    it('注册用户', function(done) {
      co(function*() {
        let userData = yield register.addUser({
          phone: phone,
          nickname: 'seon',
          password: 'asdqwe123'
        });
        assert.equal(userData.nickname, 'seon');
      }).then(done, done);
    });

    it('重复注册用户', function(done) {
      co(function*() {
        let err;
        try {
          yield register.addUser({
            phone: phone,
            nickname: 'seon',
            password: 'asdqwe123'
          });
        } catch (e) {
          err = e;
        }
        assert.equal(err.code, 3);
      }).then(done, done);
    });

  });

});
