'use strict';

var Crud = require('bem-crud');
var thunkify = require('thunkify');
var co = require('co');
var util = require('util');
var Captcha = require('bem-captcha');
var checkLoginId = require('bem-check-login-id');
var checkPassword = require('bem-check-password');
var sha1 = require('bem-sha1');

var registers = {};

class Register {
  constructor(config) {
    this.db = config.db;
    this.collection = config.collection;
    this.crud = config.crud;
  }

  static * get(config) {
    config.name = config.name || `${config.db}&${config.collection}`;
    let register = registers[config.name];
    if (!register) {
      register = registers[config.name] = new Register(config);
    }
    yield register.connect();
    return register;
  }

  * connect() {
    if (!this.connection) {
      this.connection = yield MongodbConnection.get({
        db: this.db
      });
    }
  }

  * isValid(loginId) {
    let collection = this.connection.collection(this.collection);
    let count = yield thunkify(collection.count.bind(collection))({
      loginId: loginId
    });
    let cursor = yield thunkify(collection.find.bind(collection))();
    let user = yield thunkify(cursor.toArray.bind(cursor))();
    return count === 0;
  }

  * getCaptcha(loginId) {
    let captcha = yield Captcha.get();
    return yield captcha.getCaptcha(loginId);
  }

  * addUser(captchaNum, userData) {
    yield this._checkCaptcha(captchaNum, userData);
    this._checkLoginId(userData);
    this._checkPassword(userData);
    yield this._checkExists(userData);
    userData.password = sha1(userData.password);
    let collection = this.connection.collection(this.collection);
    yield thunkify(collection.insert.bind(collection))(userData);
  }

  * _checkCaptcha(captchaNum, userData) {
    let captcha = yield Captcha.get();
    let isCaptchaValid = yield captcha.verifyCaptcha(userData.loginId, captchaNum);
    if (!isCaptchaValid) {
      throw new Error('invalidCaptcha');
    }
  }

  _checkLoginId(userData) {
    let isLoginIdValid = checkLoginId(userData.loginId);
    if (!isLoginIdValid) {
      throw new Error('invalidLoginId');
    }
  }

  _checkPassword(userData) {
    let passwordError = checkPassword(userData.password);
    if (passwordError) {
      throw new Error('invalidPassword');
    }
  }

  * _checkExists(userData) {
    let isLoginIdExits = yield this.isValid(userData.loginId);
    if (!isLoginIdExits) {
      throw new Error('existsLoginId');
    }
  }
}

module.exports = Register;
